<?php
class Quote {
    private $conn;
    private $table_name = "quotes";

    public $id;
    public $quote;
    public $author_id;
    public $category_id;

    public $error;

    public function __construct($db) {
        $this->conn = $db;
    }

    // Read all quotes
    public function read() {
        $query = "SELECT id, quote, author_id, category_id 
                  FROM " . $this->table_name . " 
                  ORDER BY id ASC";
        $stmt = $this->conn->prepare($query);
        $stmt->execute();
        return $stmt;
    }

    public function readSingle() {
        $query = "SELECT id, quote, author_id, category_id 
                  FROM " . $this->table_name . " 
                  WHERE id = ? 
                  LIMIT 0,1";
    
        $stmt = $this->conn->prepare($query);
    
        $stmt->bindValue(1, $this->id);
    
        $stmt->execute();
    
        return $stmt;
    }

    // Create a new quote
    public function create() {
        // Insert new quote
        $query = "INSERT INTO quotes (quote, author_id, category_id) VALUES (?, ?, ?)";
        $stmt = $this->conn->prepare($query);
        $stmt->bindParam(1, $this->quote);
        $stmt->bindParam(2, $this->author_id);
        $stmt->bindParam(3, $this->category_id);
    
        if ($stmt->execute()) {
            return array("success" => true, "message" => $this->conn->lastInsertId());
        }
    
        return array("success" => false, "message" => "Unable to create quote.");
    }
    
    // Update an existing quote
    public function update() {
        // Check if author_id exists in authors table
        $query = "SELECT id FROM authors WHERE id = ?";
        $stmt = $this->conn->prepare($query);
        $stmt->bindParam(1, $this->author_id);
        $stmt->execute();
        if ($stmt->rowCount() == 0) {
            return array("success" => false, "message" => "author_id not found");
        }
    
        // Check if category_id exists in categories table
        $query = "SELECT id FROM categories WHERE id = ?";
        $stmt = $this->conn->prepare($query);
        $stmt->bindParam(1, $this->category_id);
        $stmt->execute();
        if ($stmt->rowCount() == 0) {
            return array("success" => false, "message" => "category_id not found");
        }
    
        // Update quote
        $query = "UPDATE " . $this->table_name . " 
                  SET quote=:quote, author_id=:author_id, category_id=:category_id
                  WHERE id=:id";
        $stmt = $this->conn->prepare($query);
    
        // sanitize
        $this->quote = htmlspecialchars(strip_tags($this->quote));
        $this->author_id = htmlspecialchars(strip_tags($this->author_id));
        $this->category_id = htmlspecialchars(strip_tags($this->category_id));
        $this->id = htmlspecialchars(strip_tags($this->id));
    
        // bind values
        $stmt->bindParam(":quote", $this->quote);
        $stmt->bindParam(":author_id", $this->author_id);
        $stmt->bindParam(":category_id", $this->category_id);
        $stmt->bindParam(":id", $this->id);
    
        if ($stmt->execute()) {
            return array("success" => true, "message" => $this->conn->lastInsertId());
        }
        return array("success" => false, "message" => "Unable to create quote.");
    }

    // Delete a quote
    public function delete() {
        // Check if id exists
        $query = "SELECT id FROM " . $this->table_name . " WHERE id = ?";
        $stmt = $this->conn->prepare($query);
        $stmt->bindParam(1, $this->id);
        $stmt->execute();
        if ($stmt->rowCount() == 0) {
            return array("success" => false, "message" => "No Quotes Found"); // id not found
        }
    
        // Delete quote
        $query = "DELETE FROM " . $this->table_name . " WHERE id=:id";
        $stmt = $this->conn->prepare($query);
    
        // sanitize
        $this->id = htmlspecialchars(strip_tags($this->id));
    
        // bind id of record to delete
        $stmt->bindParam(":id", $this->id);
    
        if ($stmt->execute()) {
            return array("success" => true, "message" => "Quote was deleted.");
        }
        return array("success" => false, "message" => "Unable to delete quote.");
    }

    public function getQuotesByAuthorId($author_id) {
        $query = "SELECT quotes.id, quotes.quote, quotes.author_id, authors.author, quotes.category_id, categories.category
                  FROM quotes 
                  INNER JOIN authors ON quotes.author_id = authors.id 
                  INNER JOIN categories ON quotes.category_id = categories.id
                  WHERE quotes.author_id = ?";
    
        $stmt = $this->conn->prepare($query);
        $stmt->bindParam(1, $author_id);
        $stmt->execute();
    
        return $stmt;
    }
    public function getQuotesByCategoryId($category_id) {
        $query = "SELECT quotes.id, quotes.quote, quotes.author_id, quotes.category_id, categories.category 
                  FROM quotes 
                  INNER JOIN categories ON quotes.category_id = categories.id 
                  WHERE quotes.category_id = ?";
    
        $stmt = $this->conn->prepare($query);
        $stmt->bindParam(1, $category_id);
        $stmt->execute();
    
        return $stmt;
    }
    public function readByAuthorAndCategory($author_id, $category_id) {
        $query = "SELECT q.id, q.quote, a.author as author, c.category as categories 
                  FROM quotes q 
                  INNER JOIN authors a ON q.author_id = a.id 
                  INNER JOIN categories c ON q.category_id = c.id 
                  WHERE q.author_id = ? AND q.category_id = ?";
    
        $stmt = $this->conn->prepare($query);
        $stmt->bindParam(1, $author_id);
        $stmt->bindParam(2, $category_id);
        $stmt->execute();
    
        return $stmt;
    }

    public function find($id) {
        $query = "SELECT * FROM quotes WHERE id = :id LIMIT 1";
        $stmt = $this->conn->prepare($query);
        $stmt->bindParam(':id', $id);
        $stmt->execute();

        if ($stmt->rowCount() > 0) {
            return true;
        } else {
            return false;
        }
    }
}
